unit COMPort_U;
// This is a simple little class to use serial ports
// Written: 26th June 1999

// Credit due to Peter Johnson (pjohnson@netspace.net.au) for his article on
// how to use serial ports under Delphi that can be found at:
// http://www.innotts.co.uk/~zephyr/serial.html
// Thanks Pete!

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TCOMPort = class
  private
    FActive: boolean;
    ComPortFileHandle: THandle;
    function  OpenCOMPort(port: integer): THandle;
  public
    constructor Create();
    property  Active: boolean read FActive write FActive default FALSE;
    procedure GetPorts(ports: TStringList);
    function  SetupCOMPort(port: integer): boolean;
    function  ReadData(var dataRead: string; bytesToRead: integer): integer;
    function  WriteData(dataWrite: string): boolean;
    function  Test(): boolean;
    procedure CloseCOMPort();
  end;

implementation

constructor TCOMPort.Create();
begin
  inherited;
  FActive := FALSE;
  ComPortFileHandle := INVALID_HANDLE_VALUE;
end;

procedure TCOMPort.GetPorts(ports: TstringList);
var
  i: integer;
  portOpened: THandle;
begin
  for i:=1 to 4 do
    begin
    portOpened := OpenCOMPort(i);
    if portOpened<>INVALID_HANDLE_VALUE then
      begin
      ports.add('COM'+inttostr(i));
      CloseHandle(portOpened);
      end;

    end;

end;

function TCOMPort.OpenCOMPort(port: integer): THandle;
var
  portString: string;
begin
  CloseCOMPort();

  portString := 'COM'+inttostr(port);
  Result := CreateFile(PChar(portString),
                       GENERIC_READ or GENERIC_WRITE,
                       0,
                       Nil,
                       OPEN_EXISTING,
                       FILE_ATTRIBUTE_NORMAL,
                       0);

end;

function TCOMPort.SetupCOMPort(port: integer): boolean;
const
  RxBufferSize = 256;
  TxBufferSize = 256;
var
  DCB: TDCB;
  Config : String;
  CommTimeouts : TCommTimeouts;
  comPortString: string;
begin
  Result := FALSE;
  comPortString := 'COM'+inttostr(port);

  ComPortFileHandle := OpenCOMPort(port);
  if ComPortFileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('Unable to open port '+comPortString);
    exit;
    end;

  if not SetupComm(ComPortFileHandle, RxBufferSize, TxBufferSize) then
    begin
    CloseCOMPort();
    showmessage('Unable to setup port '+comPortString);
    exit;
    end;

  if not GetCommState(ComPortFileHandle, DCB) then
    begin
    showmessage('Unable to get comm state, port '+comPortString);
    CloseCOMPort();
    exit;
    end;

  Config := 'baud=19200 parity=n data=8 stop=1' + #0;

  if not BuildCommDCB(@Config[1], DCB) then
    begin
    showmessage('Unable to build DCB, port '+comPortString);
    CloseCOMPort();
    exit;
    end;

  if not SetCommState(ComPortFileHandle, DCB) then
    begin
    showmessage('Unable to set port state, port '+comPortString);
    CloseCOMPort();
    exit;
    end;

  with CommTimeouts do
    begin
    ReadIntervalTimeout := 0;
    ReadTotalTimeoutMultiplier := 0;
    ReadTotalTimeoutConstant := 100;
    WriteTotalTimeoutMultiplier := 0;
    WriteTotalTimeoutConstant := 100;
    end;

  if not SetCommTimeouts(ComPortFileHandle, CommTimeouts) then
    begin
    showmessage('Unable to set comm timeouts for port '+comPortString);
    CloseCOMPort();
    exit;
    end;

  Active := TRUE;
  Result := TRUE;
end;

function TCOMPort.ReadData(var dataRead: string; bytesToRead: integer): integer;
var
  aByte: byte;
  i: integer;
  bytesRead: integer;
begin
  Result := -1;

  if ComPortFileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('COM port not opened');
    exit;
    end;

  dataRead := '';
  for i := 1 to bytesToRead do
    begin
    if not(ReadFile(ComPortFileHandle, aByte, 1, LongWord(bytesRead), nil)) then
      begin
      showmessage('Unable to read from port');
      CloseCOMPort();
      exit;
      end;
    if bytesRead<1 then
      begin
      break;
      end;

    dataRead := dataRead + char(aByte);
    end;

  Result := length(dataRead);
end;

function TCOMPort.WriteData(dataWrite: string): boolean;
var
  i: integer;
  bytesRead: integer;
begin
  Result := FALSE;

  if ComPortFileHandle=INVALID_HANDLE_VALUE then
    begin
    showmessage('COM port not opened');
    exit;
    end;

  for i := 0 to length(dataWrite)-1 do
    begin
    if not(WriteFile(ComPortFileHandle, dataWrite[i+1], 1, LongWord(bytesRead), nil)) then
      begin
      exit;
      end;
    end;

  Result := TRUE;
end;

procedure TCOMPort.CloseCOMPort();
begin
  if ComPortFileHandle<>INVALID_HANDLE_VALUE then
    begin
    CloseHandle(ComPortFileHandle);
    ComPortFileHandle:=INVALID_HANDLE_VALUE;
    end;

  Active := FALSE;
end;

function TCOMPort.Test(): boolean;
var
  testStringOut: string;
  testStringIn: string;
begin
  Result := FALSE;

  if not(Active) then
    begin
    raise Exception.Create('COM port object not active');
    exit;
    end;

  randomize();
  testStringOut:= char(random(256));
  testStringIn:= 'WRONG!';

  if WriteData(testStringOut) then
    begin
    ReadData(testStringIn, 1);
    if testStringIn=testStringout then
      begin
      Result := TRUE;
      end;
    end;

end;

END.

