unit OptionsGeneral_U;
// Description: General Options Dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Spin,
  Menus, // Menus needed for ShortCutToText and TextToShortCut
  OptionsSelfD_U, Grids, CheckLst,
  HashAlgUnified_U;

type
  // The ordinal values of this type are used to find controls with tag=ord(x)
  TSecureTrayUtilIcon = (stuitUnmounted, stuitMounted, stuitUnsure);

type
  TOptionsGeneral_F = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    pbApply: TButton;
    pcOptions: TPageControl;
    tsGeneral: TTabSheet;
    tsHotKeys: TTabSheet;
    OpenDialog: TOpenDialog;
    GroupBox5: TGroupBox;
    Label4: TLabel;
    hkExit: THotKey;
    ckEnableExit: TCheckBox;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    hkScreenSaver: THotKey;
    ckEnableScreensaver: TCheckBox;
    tsShExt: TTabSheet;
    GroupBox4: TGroupBox;
    lblAppPath: TLabel;
    Label1: TLabel;
    ckRegShredDir: TCheckBox;
    ckRegShredFile: TCheckBox;
    ckRegMountVol: TCheckBox;
    ckRegAppPath: TCheckBox;
    ckShExtSubMenu: TCheckBox;
    ckShExtSeparators: TCheckBox;
    edShExtSubMenuTitle: TEdit;
    GroupBox3: TGroupBox;
    ckSTUOnStartUp: TCheckBox;
    ckAddToPrograms: TCheckBox;
    ckDeskSTU: TCheckBox;
    ckRegShredFreeSpace: TCheckBox;
    tsTouchFiles: TTabSheet;
    ckTouchOnStartup: TCheckBox;
    ckTouchOnExit: TCheckBox;
    ckTouchRegularly: TCheckBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    seTouchFreq: TSpinEdit;
    GroupBox7: TGroupBox;
    sgrTouchTargets: TStringGrid;
    pbAdd: TButton;
    pbEdit: TButton;
    pbDelete: TButton;
    Label10: TLabel;
    tsTaskTrayIcon: TTabSheet;
    GroupBox8: TGroupBox;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    pnlIconDisplay: TPanel;
    imgIconUnmounted: TImage;
    imgIconMounted: TImage;
    imgIconUnsure: TImage;
    TabSheet1: TTabSheet;
    Label16: TLabel;
    Label3: TLabel;
    ckDeskShred: TCheckBox;
    ckSendToShred: TCheckBox;
    Label17: TLabel;
    tsShredder: TTabSheet;
    GroupBox1: TGroupBox;
    pnlShredMethodSelect: TPanel;
    Label14: TLabel;
    Label15: TLabel;
    cbUseShredFileDir: TComboBox;
    cbUseShredDriveFree: TComboBox;
    pnlShredConfigButtons: TPanel;
    pbConfigShredDriveFree: TButton;
    pbConfigShredFileDir: TButton;
    gbWipeSwapfile: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    ckNTWipeSwapfile: TCheckBox;
    GroupBox6: TGroupBox;
    ckStartupDispVolInfo: TCheckBox;
    ckDispTrayIcon: TCheckBox;
    Label18: TLabel;
    ckRegTouch: TCheckBox;
    ckRegHash: TCheckBox;
    tsHashes: TTabSheet;
    GroupBox9: TGroupBox;
    Label19: TLabel;
    ckHashesInUppercase: TCheckBox;
    GroupBox10: TGroupBox;
    ckUseAltTasktrayMenu: TCheckBox;
    pbChangeUnmounted: TButton;
    pbChangeMounted: TButton;
    pbChangeUnsure: TButton;
    clbHashes: TCheckListBox;
    Bevel1: TBevel;
    procedure pbOKClick(Sender: TObject);
    procedure pbApplyClick(Sender: TObject);
    procedure SetApplyButton(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbAddClick(Sender: TObject);
    procedure pbEditClick(Sender: TObject);
    procedure pbDeleteClick(Sender: TObject);
    procedure UpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pbConfigShredClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CheckStartupOptsOK(Sender: TObject);
    procedure pbChangeUnmountedClick(Sender: TObject);
    procedure pbChangeMountedClick(Sender: TObject);
    procedure pbChangeUnsureClick(Sender: TObject);
  private
    hashObj: THashAlgUnified;

    TrayIcons: TImageList;
    NTWipeSwapfileOrigSetting: boolean;
    
    procedure LoadINIValues();
    procedure RegisterAppPath(reg: boolean);
    procedure RegisterShellSupport();
    procedure EnableDisableAllControls();
    procedure CreateShortcuts();
    procedure CheckShortCutsPresent();
    procedure DeleteTouchEntryRow(deleteRow: integer);
    procedure LoadIconsFromResources();
    procedure RefreshIcons();
    procedure ChooseTaskTrayIcon(imgIconUnmounted: TImage);
  public
    { Public declarations }
  end;

const
  REGKEY_NT_SHUTDOWN_WIPE_SWAPFILE       = 'SYSTEM\CurrentControlSet\Control\Session Manager';
  REGKEY_VALUE_NT_SHUTDOWN_WIPE_SWAPFILE = 'ClearPageFileAtShutdown';

  LNK_SHREDDER       = 'Shredder.lnk';
  LNK_SECURETRAYUTIL = 'SecureTrayUtil.lnk';

implementation

{$R *.DFM}
//{$R IconOptions.dcr}

uses AppGlobal_U, AppGlobalConsts_U,
     STUIniFile_U,
     Registry, RegStr,
     ConfigShredderInt_U, ConfigShredderExt_U,
     ShortCut,
     Math,
     TouchFileSelect_U,
     SDUGeneral,
     IconChooser_U;

procedure TOptionsGeneral_F.pbOKClick(Sender: TObject);
begin
  if pbApply.enabled then
    begin
    pbApplyClick(sender);
    end;

  modalresult := mrOK;

end;

procedure TOptionsGeneral_F.pbApplyClick(Sender: TObject);
var
  registry: TRegistry;
  i: integer;
  touchFileDetailsObj: TTouchFileTarget;
  currHashType: fhHashType;
begin
  // General page...
  STU_INI_FILE.OptShowVolInfoOnStartup := ckStartupDispVolInfo.checked;
  STU_INI_FILE.OptUseTaskTrayIcon := ckDispTrayIcon.checked;

  CreateShortcuts();

  if (Win32Platform=VER_PLATFORM_WIN32_NT) then
    begin
    if ckNTWipeSwapfile.checked<>NTWipeSwapfileOrigSetting then
      begin
      registry := TRegistry.create();
      try
        registry.RootKey := HKEY_LOCAL_MACHINE;
        try
          registry.OpenKey(REGKEY_NT_SHUTDOWN_WIPE_SWAPFILE, TRUE);

          if ckNTWipeSwapfile.checked then
            begin
            registry.WriteInteger(REGKEY_VALUE_NT_SHUTDOWN_WIPE_SWAPFILE, 1);
            end
          else
            begin
            registry.DeleteValue(REGKEY_VALUE_NT_SHUTDOWN_WIPE_SWAPFILE);
            end;

          registry.CloseKey();

          except
            on ERegistryException do
              begin
              showmessage('An error occured while trying to set/delete the value of "'+REGKEY_VALUE_NT_SHUTDOWN_WIPE_SWAPFILE+'"'+CRLF+
                          'for registry key'+CRLF+CRLF+
                          'HKLM\'+REGKEY_NT_SHUTDOWN_WIPE_SWAPFILE+CRLF+CRLF+
                          'Please ensure that you have the necessary permissions do perform this operation'+CRLF+
                          '(i.e. logon as administrator) and try again');
              end;
          end;
      finally
        registry.Free();
      end;
      
      end;

    end;


  // Icons page...
  STU_INI_FILE.OptTaskTrayIconUnmounted := imgIconUnmounted.tag+1;
  STU_INI_FILE.OptTaskTrayIconMounted   := imgIconMounted.tag+1;
  STU_INI_FILE.OptTaskTrayIconUnsure    := imgIconUnsure.tag+1;
  STU_INI_FILE.OptTaskTrayAlternateMenu := ckUseAltTasktrayMenu.checked;

  // Hotkeys page...
  STU_INI_FILE.OptHKeyScreenSaverEnable := ckEnableScreensaver.checked;
  STU_INI_FILE.OptHKeyScreenSaverKey    := ShortCutToText(hkScreenSaver.hotkey);
  STU_INI_FILE.OptHKeyExitEnable := ckEnableExit.checked;
  STU_INI_FILE.OptHKeyExitKey    := ShortCutToText(hkExit.hotkey);

  // Shredding page...
  STU_INI_FILE.OptShredFileDirUseInt := (cbUseShredFileDir.itemindex=0);
  STU_INI_FILE.OptShredFreeUseInt    := (cbUseShredDriveFree.itemindex=0);

  // Touching page...
  STU_INI_FILE.OptTouchFrequency := seTouchFreq.value;
  STU_INI_FILE.OptTouchRegularly := ckTouchRegularly.checked;
  STU_INI_FILE.OptTouchOnStart   := ckTouchOnStartup.checked;
  STU_INI_FILE.OptTouchOnExit    := ckTouchOnExit.checked;


  STU_INI_FILE.ClearTouchTargetFilesList();
  for i:=1 to (sgrTouchTargets.rowcount-1) do
    begin
    if sgrTouchTargets.cells[0, i]<>'' then
      begin
      touchFileDetailsObj:= TTouchFileTarget.Create();

      touchFileDetailsObj.filename := sgrTouchTargets.cells[0, i];
      if sgrTouchTargets.cells[1, i]='' then
        begin
        touchFileDetailsObj.useCurrentDateTime := TRUE;
        end
      else
        begin
        touchFileDetailsObj.useCurrentDateTime := FALSE;
        touchFileDetailsObj.datetime := strtodatetime(sgrTouchTargets.cells[1, i]);
        end;

      STU_INI_FILE.OptTouchTargetFiles.addobject(touchFileDetailsObj.filename,
                                                 touchFileDetailsObj);

      end;
    end;


  // Shell Extensions page...
  STU_INI_FILE.OptShellExtSubMenu         := ckShExtSubMenu.checked;
  STU_INI_FILE.OptShellExtSubMenuTitle    := edShExtSubMenuTitle.text;

  STU_INI_FILE.OptShellExtMenuSeparators  := ckShExtSeparators.checked;
  STU_INI_FILE.OptShellExtMenuMountVolume := ckRegMountVol.checked;
  STU_INI_FILE.OptShellExtMenuShredDir    := ckRegShredDir.checked;
  STU_INI_FILE.OptShellExtMenuShredFile   := ckRegShredFile.checked;
  STU_INI_FILE.OptShellExtMenuShredFreeSpace:= ckRegShredFreeSpace.checked;
  STU_INI_FILE.OptShellExtMenuTouch       := ckRegTouch.checked;
  STU_INI_FILE.OptShellExtMenuHash        := ckRegHash.checked;
  STU_INI_FILE.OptShellExtAppPath         := ckRegAppPath.checked;

  // Hashes page...
  for currHashType:=low(fhHashType) to high(fhHashType) do
    begin
    STU_INI_FILE.OptHashEnabled[currHashType] := TRUE;
    hashObj.ActiveHash := currHashType;
    for i:=0 to (clbHashes.items.count-1) do
      begin
      if (clbHashes.items[i] = hashObj.Title) then
        begin
        STU_INI_FILE.OptHashEnabled[currHashType] := clbHashes.Checked[i];
        break;
        end;

      end;

    end;

  STU_INI_FILE.OptHashInUppercase := ckHashesInUppercase.checked;

  // Finish off...
  STU_INI_FILE.WriteIniFile;
  pbApply.enabled := FALSE;

  LoadINIValues();

  RegisterAppPath(ckRegAppPath.checked);
  // RegisterShellSupport can screw things up (like the value of some of
  // the checkboxes; they become "Illegal" for some reason.)
  // i.e. This must be the last line in this procedure
  // Possibly fixed 2/6/01 by changing order of code in this function?
  RegisterShellSupport();

end;

procedure TOptionsGeneral_F.LoadINIValues();
var
  registry: TRegistry;
  tempInt: integer;
  currentTouchTarget: TTouchFileTarget;
  i: integer;
  currHashType: fhHashType;
begin
  STU_INI_FILE.ReadIniFile;

  // General page...
  ckStartupDispVolInfo.checked := STU_INI_FILE.OptShowVolInfoOnStartup;
  ckDispTrayIcon.checked := STU_INI_FILE.OptUseTaskTrayIcon;

  CheckShortCutsPresent();

  tempInt := 0;
  if (Win32Platform=VER_PLATFORM_WIN32_NT) then
    begin
    registry := TRegistry.create();
    try
      registry.RootKey := HKEY_LOCAL_MACHINE;
      if registry.OpenKey(REGKEY_NT_SHUTDOWN_WIPE_SWAPFILE, FALSE) then
        begin
        try
          tempInt := registry.ReadInteger(REGKEY_VALUE_NT_SHUTDOWN_WIPE_SWAPFILE);
        except
          on ERegistryException do
            begin
            // Nothing
            end;
        end;
        registry.CloseKey();
        end;
    finally
      registry.Free();
    end;

    end;
  ckNTWipeSwapfile.checked := (tempInt=1);
  NTWipeSwapfileOrigSetting := ckNTWipeSwapfile.checked;

  // Icons page...
  imgIconUnmounted.tag := STU_INI_FILE.OptTaskTrayIconUnmounted-1;
  imgIconMounted.tag   := STU_INI_FILE.OptTaskTrayIconMounted-1;
  imgIconUnsure.tag    := STU_INI_FILE.OptTaskTrayIconUnsure-1;
  ckUseAltTasktrayMenu.checked := STU_INI_FILE.OptTaskTrayAlternateMenu;

  // Hotkeys page...
  ckEnableScreensaver.checked := STU_INI_FILE.OptHKeyScreenSaverEnable;
  hkScreenSaver.hotkey        := TextToShortCut(STU_INI_FILE.OptHKeyScreenSaverKey);
  ckEnableExit.checked := STU_INI_FILE.OptHKeyExitEnable;
  hkExit.hotkey        := TextToShortCut(STU_INI_FILE.OptHKeyExitKey);

  // Shredder page
  cbUseShredFileDir.itemindex := 0;
  if not(STU_INI_FILE.OptShredFileDirUseInt) then
    begin
    cbUseShredFileDir.itemindex := 1;
    end;

  cbUseShredDriveFree.itemindex := 0;
  if not(STU_INI_FILE.OptShredFreeUseInt) then
    begin
    cbUseShredDriveFree.itemindex := 1;
    end;

  // Touching page...
  seTouchFreq.value := STU_INI_FILE.OptTouchFrequency;
  ckTouchRegularly.checked := STU_INI_FILE.OptTouchRegularly;
  ckTouchOnStartup.checked := STU_INI_FILE.OptTouchOnStart;
  ckTouchOnExit.checked    := STU_INI_FILE.OptTouchOnExit;

  
  sgrTouchTargets.rowcount:=2;
  for i:=0 to (STU_INI_FILE.OptTouchTargetFiles.count-1) do
    begin
    if i<>0 then
      begin
      sgrTouchTargets.rowcount := sgrTouchTargets.rowcount + 1;
      end;

    currentTouchTarget := TTouchFileTarget(STU_INI_FILE.OptTouchTargetFiles.objects[i]);

    sgrTouchTargets.cells[0, (sgrTouchTargets.rowcount-1)] := currentTouchTarget.filename;

    if currentTouchTarget.useCurrentDateTime then
      begin
      sgrTouchTargets.cells[1, (sgrTouchTargets.rowcount-1)] := '';
      end
    else
      begin
      sgrTouchTargets.cells[1, (sgrTouchTargets.rowcount-1)] := datetimetostr(currentTouchTarget.datetime);
      end;

    end;

  // Shell Extensions page...
  ckRegMountVol.checked  := STU_INI_FILE.OptShellExtMenuMountVolume;
  ckRegShredDir.checked  := STU_INI_FILE.OptShellExtMenuShredDir;
  ckRegShredFile.checked := STU_INI_FILE.OptShellExtMenuShredFile;
  ckRegShredFreeSpace.checked := STU_INI_FILE.OptShellExtMenuShredFreeSpace;
  ckRegTouch.checked     := STU_INI_FILE.OptShellExtMenuTouch;
  ckRegHash.checked      := STU_INI_FILE.OptShellExtMenuHash;
  ckRegAppPath.checked   := STU_INI_FILE.OptShellExtAppPath;

  // These three must come *after* the others on this page, as they
  // are affected by them
  ckShExtSubMenu.checked := STU_INI_FILE.OptShellExtSubMenu;
  edShExtSubMenuTitle.text := STU_INI_FILE.OptShellExtSubMenuTitle;
  ckShExtSeparators.checked := STU_INI_FILE.OptShellExtMenuSeparators;

  // Hashes page...
  for currHashType:=low(fhHashType) to high(fhHashType) do
    begin
    hashObj.ActiveHash := currHashType;
    for i:=0 to (clbHashes.items.count-1) do
      begin
      if (clbHashes.items[i] = hashObj.Title) then
        begin
        clbHashes.Checked[i] := STU_INI_FILE.OptHashEnabled[currHashType];
        break;
        end;

      end;

    end;

  ckHashesInUppercase.checked := STU_INI_FILE.OptHashInUppercase;

  // Finish off...
  pbApply.enabled := FALSE;
end;

procedure TOptionsGeneral_F.SetApplyButton(Sender: TObject);
begin
  EnableDisableAllControls();
  pbApply.enabled := TRUE;

end;

procedure TOptionsGeneral_F.FormCreate(Sender: TObject);
var
  currHashType: fhHashType;
begin
  hashObj:= THashAlgUnified.Create(nil);

  for currHashType:=low(fhHashType) to high(fhHashType) do
    begin
    hashObj.ActiveHash := currHashType;
    clbHashes.Items.Add(hashObj.Title);
    end;

  sgrTouchTargets.DefaultRowHeight := sgrTouchTargets.font.size*2;
  sgrTouchTargets.cells[0, 0] := 'File/directory name';
  sgrTouchTargets.cells[1, 0] := 'Date/Time';

  pnlIconDisplay.bevelouter := bvNone;

  TrayIcons := TImageList.Create(nil);
  TrayIcons.height := 32;
  TrayIcons.width := 32;

  LoadIconsFromResources();

  LoadINIValues();
  RefreshIcons();

  pcOptions.activepage := tsGeneral;
end;


// Register/Unregister the application's path
procedure TOptionsGeneral_F.RegisterAppPath(reg: boolean);
var
  registry: TRegistry;
  keyName: String;
  exePath: string;
begin
  registry := TRegistry.Create;
  try
    registry.RootKey := HKEY_LOCAL_MACHINE;
    keyName := REGSTR_PATH_APPPATHS + '\SecureTrayUtil.exe';
    if reg then
      begin
      try
        registry.OpenKey(keyName, True);
        // Application.ExeName must = '<path>/SecureTrayUtil.exe' for this to
        // work with the ShellExt DLL
        registry.WriteString('', Application.ExeName);
        registry.CloseKey;
        registry.OpenKey(keyName, True);
        exePath := ExtractFilePath(Application.ExeName);
        Delete(exePath, Length(exePath), 1);
        registry.WriteString('Path', exePath);
        registry.CloseKey;
      except
        on ERegistryException do
          begin
          showmessage('An error occured while writing to the registry; if you are running under NT,'+CRLF+
                      'please ensure that you have the necessary permissions do perform this operation'+CRLF+
                      '(i.e. logon as administrator) and try again');
          end;
      end;
      end
    else
      begin
      registry.DeleteKey(keyName);
      end;
  finally
    registry.Free;
  end;
end;


procedure TOptionsGeneral_F.RegisterShellSupport();
var
  regDLL: boolean;
  Handle:  THandle;
//  RegProc: function(r: boolean): HResult; stdcall;
  RegProc: function(): HResult; stdcall;
  hr: HResult;
  callProc: string;
begin
  // Register if any one of the shell items is TRUE, and the AppPath is TRUE
  regDLL := ( (STU_INI_FILE.OptShellExtMenuMountVolume)     OR
              (STU_INI_FILE.OptShellExtMenuShredFreeSpace)  OR
              (STU_INI_FILE.OptShellExtMenuShredDir)        OR
              (STU_INI_FILE.OptShellExtMenuShredFile)       OR
              (STU_INI_FILE.OptShellExtMenuTouch)           OR
              (STU_INI_FILE.OptShellExtMenuHash) )          AND
            (STU_INI_FILE.OptShellExtAppPath);

  if regDLL then
    begin
    callProc := 'DllRegisterServer';
    end
  else
    begin
    callProc := 'DllUnregisterServer';
    end;

  // Call Register on the DLL with the value of reg
  Handle := LoadLibrary(SHELL_EXT_FILENAME);
  if Handle=0 then
    begin
    raise Exception.CreateFmt('%s: %s',
                              [SysErrorMessage(GetLastError),
                               SHELL_EXT_FILENAME]);
    end;
  try
    RegProc := GetProcAddress(Handle, PChar(callProc));
    if Assigned(RegProc) then
      begin
      hr := RegProc();
      if Failed(hr) then
        begin
        raise Exception.Create(callProc+' in '+SHELL_EXT_FILENAME+' failed.')
        end;
      end
    else
      begin
      RaiseLastOSError;
      end;
  finally
    FreeLibrary(Handle)
  end

end;


procedure TOptionsGeneral_F.EnableDisableAllControls();
var
  appPathEnabled: boolean;
begin
  SDUEnableControl(gbWipeSwapfile, (Win32Platform=VER_PLATFORM_WIN32_NT));

  appPathEnabled := TRUE;
  if (ckRegMountVol.checked)  OR
     (ckRegShredFile.checked) OR
     (ckRegShredDir.checked)  OR
     (ckRegShredFreeSpace.checked) OR
     (ckRegTouch.checked)     OR
     (ckRegHash.checked)      then
    begin
    ckRegAppPath.checked := TRUE;
    appPathEnabled := FALSE;
    end;

  SDUEnableControl(ckRegAppPath, appPathEnabled);
  SDUEnableControl(lblAppPath, appPathEnabled);
  SDUEnableControl(ckShExtSubMenu, not(appPathEnabled));
  SDUEnableControl(edShExtSubMenuTitle, (not(appPathEnabled) AND ckShExtSubMenu.checked));
  SDUEnableControl(ckShExtSeparators, not(appPathEnabled));

  if not(ckShExtSubMenu.enabled) then
    begin
    ckShExtSubMenu.checked := FALSE;
    ckShExtSeparators.checked := FALSE;
    end;

  SDUEnableControl(hkScreenSaver, ckEnableScreenSaver.checked);
  SDUEnableControl(hkExit, ckEnableExit.checked);

  SDUEnableControl(seTouchFreq, ckTouchRegularly.checked);

end;


procedure TOptionsGeneral_F.CreateShortcuts();
begin
  if ckDeskSTU.checked then
    begin
    CreateLink(Application.ExeName,
               '',
               ExtractFilePath(Application.ExeName),
               LNK_SECURETRAYUTIL,
               slDesktop);
    end
  else
    begin
    DeleteLink(LNK_SECURETRAYUTIL, slDesktop);
    end;

  if ckSTUOnStartUp.checked then
    begin
    CreateLink(Application.ExeName,
               '',
               ExtractFilePath(Application.ExeName),
               LNK_SECURETRAYUTIL,
               slStartUp);
    end
  else
    begin
    DeleteLink(LNK_SECURETRAYUTIL, slStartUp);
    end;

  if ckAddToPrograms.checked then
    begin
    CreateLink(Application.ExeName,
               '',
               ExtractFilePath(Application.ExeName),
               LNK_SECURETRAYUTIL,
               slPrograms);
    end
  else
    begin
    DeleteLink(LNK_SECURETRAYUTIL, slPrograms);
    end;

end;


procedure TOptionsGeneral_F.CheckShortCutsPresent();
begin
//  ckSendToShred.checked   := CheckLink(LNK_SHREDDER, slSendTo);
//  ckDeskShred.checked     := CheckLink(LNK_SHREDDER, slDesktop);
  ckDeskSTU.checked       := CheckLink(LNK_SECURETRAYUTIL, slDesktop);
  ckSTUOnStartUp.checked  := CheckLink(LNK_SECURETRAYUTIL, slStartUp);
  ckAddToPrograms.checked := CheckLink(LNK_SECURETRAYUTIL, slPrograms);

end;


procedure TOptionsGeneral_F.pbAddClick(Sender: TObject);
var
  touchDetailsDlg: TTouchFileSelect_F;
begin
  touchDetailsDlg:= TTouchFileSelect_F.create(nil);
  try
    touchDetailsDlg.WarnOnNonExistantFile := TRUE;
    if touchDetailsDlg.ShowModal()=mrOK then
      begin
      if sgrTouchTargets.cells[0, 1]<>'' then
        begin
        sgrTouchTargets.rowcount := sgrTouchTargets.rowcount +1;
        end;
      sgrTouchTargets.cells[0, (sgrTouchTargets.rowcount-1)] := touchDetailsDlg.edFilename.text;
      sgrTouchTargets.cells[1, (sgrTouchTargets.rowcount-1)] := touchDetailsDlg.DateTimeAsString;

      SetApplyButton(nil);
      end;
  finally
    touchDetailsDlg.Free();
  end;

end;

procedure TOptionsGeneral_F.pbEditClick(Sender: TObject);
var
  touchDetailsDlg: TTouchFileSelect_F;
begin
  touchDetailsDlg:= TTouchFileSelect_F.create(nil);
  try
    touchDetailsDlg.WarnOnNonExistantFile := TRUE;
    touchDetailsDlg.edFilename.text := sgrTouchTargets.cells[0, sgrTouchTargets.row];
    touchDetailsDlg.DateTimeAsString := sgrTouchTargets.cells[1, sgrTouchTargets.row];
    if touchDetailsDlg.ShowModal()=mrOK then
      begin
      sgrTouchTargets.cells[0, sgrTouchTargets.row] := touchDetailsDlg.edFilename.text;
      sgrTouchTargets.cells[1, sgrTouchTargets.row] := touchDetailsDlg.DateTimeAsString;

      SetApplyButton(nil);
      end;
  finally
    touchDetailsDlg.Free();
  end;

end;

procedure TOptionsGeneral_F.pbDeleteClick(Sender: TObject);
begin
  if messageDlg('Delete selected?', mtConfirmation, [mbYes, mbNo], 0)=mrYes then
    begin
    DeleteTouchEntryRow(sgrTouchTargets.row);
    SetApplyButton(nil);
    end;

end;

procedure TOptionsGeneral_F.DeleteTouchEntryRow(deleteRow: integer);
var
  i: integer;
begin
  if (deleteRow>0)                        AND
     (deleteRow<sgrTouchTargets.rowcount) then
    begin
    sgrTouchTargets.cells[0, deleteRow] := '';
    sgrTouchTargets.cells[1, deleteRow] := '';
    for i:=deleteRow to (sgrTouchTargets.rowcount-1) do
      begin
      sgrTouchTargets.rows[i] := sgrTouchTargets.rows[i+1];
      end;
    sgrTouchTargets.rowcount := max((sgrTouchTargets.rowcount - 1), 2);
    end;
  SetApplyButton(nil);

end;

procedure TOptionsGeneral_F.RefreshIcons();
var
  currIcon: TIcon;
begin
  currIcon:=TIcon.Create();

  TrayIcons.geticon(imgIconUnmounted.tag, currIcon);
  imgIconUnmounted.picture.icon := currIcon;

  TrayIcons.geticon(imgIconMounted.tag, currIcon);
  imgIconMounted.picture.icon := currIcon;

  TrayIcons.geticon(imgIconUnsure.tag, currIcon);
  imgIconUnsure.picture.icon := currIcon;

  currIcon.Free();

end;

procedure TOptionsGeneral_F.LoadIconsFromResources();
var
  anIcon: TIcon;
  thehIcon: hIcon;
  i: integer;
begin
  for i:=low(RES_ICON_NAMES) to high(RES_ICON_NAMES) do
    begin
    thehIcon:= LoadIcon(hInstance, PChar(RES_ICON_NAMES[i]));
    anIcon:= TIcon.Create();
    anIcon.ReleaseHandle();
    anIcon.handle := thehIcon;

    TrayIcons.addicon(anIcon);

    anIcon.ReleaseHandle();
    DestroyIcon(thehIcon);
    anIcon.Free();
    end;

end;

procedure TOptionsGeneral_F.UpDownClick(Sender: TObject;
  Button: TUDBtnType);
begin
  RefreshIcons();
  SetApplyButton(Sender);

end;

procedure TOptionsGeneral_F.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  TrayIcons.Free();
  hashObj.Free();

end;

procedure TOptionsGeneral_F.pbConfigShredClick(Sender: TObject);
var
  intConfigDlg: TConfigShredderInt_F;
  extConfigDlg: TConfigShredderExt_F;
begin
  if TComboBox(SDUGetControlWithTag(TButton(Sender).tag, pnlShredMethodSelect)).itemindex=0 then
    begin
    intConfigDlg := TConfigShredderInt_F.Create(nil);
    try
      intConfigDlg.showmodal();
    finally
      intConfigDlg.Free();
    end;
    end
  else
    begin
    extConfigDlg := TConfigShredderExt_F.Create(nil);
    try
      extConfigDlg.showmodal();
    finally
      extConfigDlg.Free();
    end;
    end;

end;

procedure TOptionsGeneral_F.FormShow(Sender: TObject);
begin
  pnlShredMethodSelect.bevelouter  := bvNone;
  pnlShredConfigButtons.bevelouter := bvNone;

end;

procedure TOptionsGeneral_F.CheckStartupOptsOK(Sender: TObject);
begin
  if not(ckDispTrayIcon.checked OR ckStartupDispVolInfo.checked) then
    begin
    showmessage('You must either display the tasktray icon, or display the main window on startup.');
    TCheckBox(Sender).Checked := TRUE;
    end;

  SetApplyButton(Sender);

end;

procedure TOptionsGeneral_F.ChooseTaskTrayIcon(imgIconUnmounted: TImage);
var
  chooserDlg: TIconChooser_F;
  i: integer;
begin
  chooserDlg:= TIconChooser_F.Create(nil);
  try
    chooserDlg.SetIcons(TrayIcons);
    for i:=low(RES_ICON_NAMES) to high(RES_ICON_NAMES) do
      begin
      chooserDlg.AddItem(RES_ICON_NAMES[i], i-1);
      end;

    chooserDlg.Selected := imgIconUnmounted.tag;
    if chooserDlg.ShowModal()=mrOK then
      begin
      imgIconUnmounted.tag := chooserDlg.Selected;
      pbApply.Enabled := TRUE;
      RefreshIcons();
      end;

  finally
    chooserDlg.Free();
  end;

end;


procedure TOptionsGeneral_F.pbChangeUnmountedClick(Sender: TObject);
begin
  ChooseTaskTrayIcon(imgIconUnmounted);

end;

procedure TOptionsGeneral_F.pbChangeMountedClick(Sender: TObject);
begin
  ChooseTaskTrayIcon(imgIconMounted);

end;

procedure TOptionsGeneral_F.pbChangeUnsureClick(Sender: TObject);
begin
  ChooseTaskTrayIcon(imgIconUnsure);

end;


END.

