unit TouchFileSelect_U;
// Description: Dialog to allow the user to specify a filename and time/date
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Buttons;

type
  TTouchFileSelect_F = class(TForm)
    edFilename: TEdit;
    Label1: TLabel;
    pbOK: TButton;
    pbCancel: TButton;
    Label2: TLabel;
    OpenDialog: TOpenDialog;
    dtpTime: TDateTimePicker;
    dtpDate: TDateTimePicker;
    rbCurrDateTime: TRadioButton;
    rbSpecifiedDateTime: TRadioButton;
    pbBrowseDir: TBitBtn;
    pbBrowseFile: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbBrowseFileClick(Sender: TObject);
    procedure pbBrowseDirClick(Sender: TObject);
  private
    fWarnOnNonExistantFile: boolean;
    fForceExistantFile: boolean;
  public
    DateTimeAsString: string; // Set to '' on exit to indicate the current time
  published
    property WarnOnNonExistantFile: boolean read fWarnOnNonExistantFile write fWarnOnNonExistantFile default TRUE;
    property ForceExistantFile: boolean read fForceExistantFile write fForceExistantFile default TRUE;
  end;

implementation

{$R *.DFM}

{$IFDEF LINUX}
This software is only intended for use under MS Windows
{$ENDIF}
{$WARN UNIT_PLATFORM OFF}  // Useless warning about platform - we're already
                           // protecting against that!
{$WARN SYMBOL_PLATFORM OFF}  // Useless warning about platform - we're already
                             // protecting against that!

uses
  AppGlobal_U, SDUGeneral,
  FileCtrl;

procedure TTouchFileSelect_F.FormShow(Sender: TObject);
begin
  rbCurrDateTime.checked := TRUE;
  if DateTimeAsString<>'' then
    begin
    rbSpecifiedDateTime.checked := TRUE;
    dtpDate.date := strtodatetime(DateTimeAsString);
    dtpTime.time := strtodatetime(DateTimeAsString);
    end;

end;

procedure TTouchFileSelect_F.pbOKClick(Sender: TObject);
var
  inputOK: boolean;
begin
  inputOK := TRUE;

  if inputOK AND
     (edFilename.text='') then
    begin
    showmessage('You must specify a file/directory to touch.');
    inputOK := FALSE;
    end
  else
    begin
    if inputOK AND
       fForceExistantFile AND
       not(FileGetAttr(edFilename.text)<>-1) then
      begin
      showmessage('File/directory "'+edFilename.text+'" not found.');
      inputOK := FALSE;
      end
    else if inputOK AND
            fWarnOnNonExistantFile AND
            not(FileGetAttr(edFilename.text)<>-1) then
      begin
      inputOK := (messageDlg('File/directory not found - continue anyway?', mtConfirmation, [mbYes, mbNo], 0)=mrYes);
      end

    end;

  if rbCurrDateTime.checked then
    begin
    DateTimeAsString := '';
    end
  else
    begin
    DateTimeAsString := datetostr(dtpDate.date)+' '+timetostr(dtpTime.time);
    end;

  if inputOK then
    begin
    ModalResult := mrOK;
    end;

end;

procedure TTouchFileSelect_F.FormCreate(Sender: TObject);
begin
  dtpDate.Date := Now();
  dtpTime.Time := Now();

  pbBrowseFile.glyph.LoadFromResourceName(hInstance, 'BITMAP_FILE');
  pbBrowseFile.caption := '';
  pbBrowseDir.glyph.LoadFromResourceName(hInstance, 'BITMAP_DIR');
  pbBrowseDir.caption := '';

//  BITMAP.Transparent := TRUE;
//   - on the options: self destruct:  targets
// ENSURE THAT THE "FILE" bitmap in the securetrayutil resource file
// has it's bottomleftmost pixel as purple, and set the background to purple
end;

procedure TTouchFileSelect_F.pbBrowseFileClick(Sender: TObject);
begin
  SDUOpenSaveDialogSetup(OpenDialog, edFilename.text);
  if OpenDialog.Execute() then
    begin
    edFilename.text := OpenDialog.Filename;
    end;

end;

procedure TTouchFileSelect_F.pbBrowseDirClick(Sender: TObject);
var
  dirToTouch: string;
begin
  if SelectDirectory('Select Directory to Touch', '', dirToTouch) then
    begin
    edFilename.text := dirToTouch;
    end;

end;

// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
{$WARN SYMBOL_PLATFORM ON}
{$WARN UNIT_PLATFORM ON}
// -----------------------------------------------------------------------------

END.



